<?php
// +----------------------------------------------------------------------+
// | OpenConf                                                             |
// +----------------------------------------------------------------------+
// | Copyright (c) 2002-2011 Zakon Group LLC.  All Rights Reserved.       |
// +----------------------------------------------------------------------+
// | This source file is subject to the OpenConf License, available on    |
// | the OpenConf web site: www.OpenConf.com                              |
// +----------------------------------------------------------------------+

// Form array format
/*
	fieldsetAR: (index - fieldset id [fs_ID])
		fieldset: Fieldset title
		note: Note to appear atop fieldset
		fields: Array of field IDs
	fieldAR: (index - field database id)
		name: Full field name
		short: Short name for use in confirmation message and review display to chair/reviewers
		note: Note to appear below field
		type: Field type to use (radio, checkbox, text, textarea, select, password, plain)
		size: text field length or textarea number of rows
		maxsize: text field max size
		blank: if type=select, blank=true will include a blank option tag
		usekey: TRUE - use values array key for checkbox/radio/select field value; FALSE - use array value or N/A (non-radio/checkbox/select)
		values: Array of radio/checkbox options or empty string if N/A
		delimiter: Delimiter to use to separate radio/checbox options (default: <br />), or next field
		enabled: TRUE (default) - field editable; FALSE - not editable (HTML "disabled" attribute included)
		required: TRUE - field required; FALSE (default) - field not required (may not be used in all instances)
	
	NOTE: All fieldset and field IDs need to be unique.  Prefix fieldset IDs with fs_ for OC, fs_[MOD]_ for modules
	
	NOTE: Fieldset fs_authors, and fields named "file" or starting with "password", have special handling
*/

$oc_defaultDelimiter = '<br />';
$oc_defaultFieldSize = '60';

if (!isset($GLOBALS['oc_authorNum'])) {
	$GLOBALS['oc_authorNum'] = $GLOBALS['OC_configAR']['OC_authorsMax'];
}

function oc_fieldEnabled($fieldID, &$fieldsAR) {
	if (!isset($fieldsAR[$fieldID]['enabled']) || $fieldsAR[$fieldID]['enabled']) {
		return(TRUE);
	} else {
		return(FALSE);
	}
}

function oc_genField(&$fid, &$fAR, &$fVals, $fidxtra='') {
	$usefid = $fid . $fidxtra;

	$field = '';

	$field .= '<div class="field">';
	
	if (isset($fAR['enabled']) && !$fAR['enabled']) {
		$disabled = 'disabled ';
	} else {
		$disabled = '';
	}
	
	if (isset($fAR['required']) && $fAR['required']) {
		$required = $GLOBALS['OC_configAR']['OC_requiredField'];
	} else {
		$required = '';
	}
	
	switch ($fAR['type']) {

		case 'plain':
			$field .= '<label>' . (empty($fAR['name']) ? '&nbsp;' : ($fAR['name'] . ':')) . '</label>' . $fAR['values'];
			break;

		case 'text':
			$field .= '<label for="' . $usefid . '">' . $fAR['name'] . ':' . $required . '</label><input name="' . $usefid . '" id="' . $usefid . '" size="' . varValue('size', $fAR, $GLOBALS['oc_defaultFieldSize']) . '" maxlength="' . varValue('maxsize', $fAR) . '" value="' . safeHTMLstr(varValue($usefid, $fVals)) . '" ' . $disabled . '/>';
			break;

		case 'textarea':
			$field .= '<label for="' . $usefid . '">' . $fAR['name'] . ':' . $required . '</label><textarea name="' . $usefid . '" id="' . $usefid . '" cols="' . $GLOBALS['oc_defaultFieldSize'] . '" rows="' . varValue('size', $fAR) . '" ' . $disabled . '>' . safeHTMLstr(varValue($usefid, $fVals)) . '</textarea>';
			if (isset($fAR['maxsize']) && ($fAR['maxsize'] > 0)) {
				$field .= '
<script language="javascript" type="text/javascript">
<!--
var oc_' . $usefid . 'Field = document.getElementById("' . $usefid . '");
var oc_transAR = new Array();
oc_transAR[\'%1$s field contains %2$d words; limit is %3$d words.\'] = \'' . preg_replace("/'/", "\\'", oc_('%1$s field contains %2$d words; limit is %3$d words.')) .  '\';
// Add blur check
if (oc_' . $usefid . 'Field.addEventListener) {
	oc_' . $usefid . 'Field.addEventListener("blur", function(){oc_checkWordNum("' . $usefid . '", "' . safeHTMLstr($fAR['short']) . '", ' . $fAR['maxsize'] . ')}, false);
} else if (oc_' . $usefid . 'Field.attachEvent) {
	oc_' . $usefid . 'Field.attachEvent("onblur", function(){oc_checkWordNum("' . $usefid . '", "' . safeHTMLstr($fAR['short']) . '", ' . $fAR['maxsize'] . ')});
}
// -->
</script>
';
			}
			break;

		case 'select':
			$field .= '<label for="' . $usefid . '">' . $fAR['name'] . ':' . $required . '</label><select name="' . $usefid . '" id="' . $usefid . '" ' . $disabled . '>' . ((isset($fAR['blank']) && $fAR['blank']) ? '<option value=""></option>' : '') . generateSelectOptions($fAR['values'], varValue($usefid, $fVals), $fAR['usekey']) . '</select>';
			break;

		case 'radio':
			$field .= '<label>' . $fAR['name'] . ':' . $required . '</label><fieldset class="radio">';
			$field .= generateRadioOptions($usefid, $fAR['values'], varValue($usefid, $fVals), $fAR['usekey'], $disabled, (isset($fAR['delimiter']) ? $fAR['delimiter'] : $GLOBALS['oc_defaultDelimiter']));
			$field .= '</fieldset>';
			break;

		case 'checkbox':
			$field .= '<label>' . $fAR['name'] . ':' . $required . '</label><fieldset class="checkbox">';
			if (isset($fVals[$usefid])) {
				if (!is_array($fVals[$usefid])) {
					$vals = explode(',', $fVals[$usefid]);
				} else {
					$vals = $fVals[$usefid];
				}
			} else {
				$vals = array();
			}
			$field .= generateCheckboxOptions($usefid, $fAR['values'], $vals, $fAR['usekey'], $disabled, (isset($fAR['delimiter']) ? $fAR['delimiter'] :  $GLOBALS['oc_defaultDelimiter']));
			$field .= '</fieldset>';
			break;

		case 'country': // DEPRECATED - use select with usekey=true and values=$OC_countryAR
			$field .= '<label for="' . $usefid . '">' . $fAR['name'] . ':' . $required . '</label><select name="' . $usefid . '" id="' . $usefid . '" ' . $disabled . '><option value=""></option>' . generateSelectOptions($GLOBALS['OC_countryAR'], varValue($usefid, $fVals), $fAR['usekey']) . '</select>';
			break;

		case 'password':
			$field .= '<label for="' . $usefid . '">' . $fAR['name'] . ':' . $required . '</label><input name="' . $usefid . '" id="' . $usefid . '" type="password" size="60" value="' . safeHTMLstr(varValue($usefid, $fVals)) . '" />';
			break;

		case 'file':
			$field .= '<label for="' . $usefid . '">' . $fAR['name'] . ':' . $required . '</label><input name="' . $usefid. '" id="' . $usefid . '" type="file" />';
			break;

		default:
			err('There is an error with field ID ' . $usefid);
	}

	if (!empty($fAR['note']) && ($fAR['type'] != 'plain')) {
		$field .= '<div class="fieldnote note">' . $fAR['note'] . '</div>';
	}
	
	$field .= '</div>';	
	
	return($field);
	
}

function oc_displayFields(&$fieldsAR, &$fVals=array()) {
	foreach ($fieldsAR as $fieldID => $fieldAR) {
		print '<div class="question">' . oc_genField($fieldID, $fieldAR, $fVals) . '</div>';
	}
}

function oc_displayFieldSet(&$fsAR, &$fieldsAR, &$fVals=array()) {
	foreach ($fsAR as $fieldset => $fieldsetAR) {
		if (count($fieldsetAR['fields']) == 0) { continue; }	// skip fieldset if no fields included
		print '<fieldset id="' . safeHTMLstr($fieldset) . '"><legend>' . safeHTMLstr($fieldsetAR['fieldset']) . '</legend>';
		if (!empty($fieldsetAR['note'])) {
			print '<div class="fieldsetnote note">' . $fieldsetAR['note'] . '</div>';
		}
		
		if ($fieldset == 'fs_authors') {
			// setup dynamic add authors
			$authorFields = '<legend>' . oc_('Author') . ' [:authornum:]</legend>';
			$blankAR = array();
			foreach ($fieldsetAR['fields'] as $fieldID) {
				if (!isset($fieldsAR[$fieldID])) { continue; }
				$authorFields .= oc_genField($fieldID, $fieldsAR[$fieldID], $blankAR, '[:authornum:]');
			}
			print '
<script language="javascript" type="text/javascript">
<!--
var oc_authorsMax = ' . $GLOBALS['OC_configAR']['OC_authorsMax'] . ';
var oc_authorsMinDisplay = ' . $GLOBALS['OC_configAR']['OC_authorsMinDisplay'] . ';
var oc_authorNum = ' . $GLOBALS['oc_authorNum'] . ';
var authorFields = ' . json_encode($authorFields) . ';
var updateAuthorFieldsAR = new Array("contactid");
function oc_addAuthor() {
	if ( ! document.getElementById ) { return; }
	if (oc_authorNum >= oc_authorsMax) { alert("' . oc_('Maximum number of authors allowed by the Chair reached') . '"); return; }
	oc_authorNum++;
	var newAuthorFields = authorFields.replace(/\[:authornum:\]/g, oc_authorNum);
	var newFS = document.createElement("fieldset");
	newFS.innerHTML = newAuthorFields;
	var addAuthorObj = document.getElementById("addauthor");
	addAuthorObj.parentNode.insertBefore(newFS, addAuthorObj);
	document.getElementById("authornum").value = oc_authorNum;
	for (var f in updateAuthorFieldsAR) {
		var fEl = document.getElementById(updateAuthorFieldsAR[f]);
		fEl.options[fEl.length] = new Option("' . oc_('Author') . ' " + oc_authorNum, oc_authorNum);
	}
	if (oc_authorNum == oc_authorsMax) { 
		addAuthorObj.style.display = "none";
	}
}
// -->
</script>
';

			// display author fieldsets
			for ($i=1; $i<=$GLOBALS['oc_authorNum']; $i++) {
				print '<fieldset><legend>' . oc_('Author') . ' ' . $i . '</legend>';
				foreach ($fieldsetAR['fields'] as $fieldID) {
					if (!isset($fieldsAR[$fieldID])) { continue; }
					print oc_genField($fieldID, $fieldsAR[$fieldID], $fVals, $i);
				}
				print '</fieldset>';
			}
			
			// display extra author link
			if ($i <= $GLOBALS['OC_configAR']['OC_authorsMax']) {
				print '
<script language="javascript" type="text/javascript">
<!--
document.write(\'<div id="addauthor"><span style="" onclick="oc_addAuthor()" title="' . htmlspecialchars(oc_('click to add more authors'), ENT_QUOTES, 'UTF-8') . '">' . htmlspecialchars(oc_('Add Author'), ENT_QUOTES, 'UTF-8') . '</span></div>\');
// -->
</script>
<noscript>
<div class="warn">' . oc_('To add more authors, enable JavaScript and reload page.') . '</div>
</noscript>
';
			}
		} else {	// not authors fieldset
			foreach ($fieldsetAR['fields'] as $fieldID) {
				if (!isset($fieldsAR[$fieldID])) { continue; }
				print oc_genField($fieldID, $fieldsAR[$fieldID], $fVals);
			}
		}
		print '</fieldset>';
	}
}

function oc_getFieldValue(&$fieldsAR, &$fVals, $fieldID, $valID='') {
	if (empty($valID)) {
		$valID = $fieldID;
	}

	if (!isset($fVals[$valID])) {
		return('');
	} elseif (is_array($fVals[$valID])) {
		if (count($fVals[$valID]) == 1) {
			if (is_array($fieldsAR[$fieldID]['values']) && $fieldsAR[$fieldID]['usekey']) {
				return($fieldsAR[$valID]['values'][$fVals[$valID][0]]);
			} else {
				return($fVals[$valID][0]);
			}
		} else {
			$fret = "\n";
			foreach ($fVals[$valID] as $f) {
				if (is_array($fieldsAR[$fieldID]['values']) && $fieldsAR[$fieldID]['usekey']) {
					$fret .= '    - ' . $fieldsAR[$valID]['values'][$f] . "\n";
				} else {
					$fret .= '    - ' . $f . "\n";
				}
			}
		}
		return($fret);
	} elseif (isset($fieldsAR[$fieldID]['values']) && is_array($fieldsAR[$fieldID]['values']) 
				&& isset($fieldsAR[$fieldID]['usekey']) && $fieldsAR[$fieldID]['usekey'] 
				&& !empty($fVals[$valID])
				&& isset($fieldsAR[$fieldID]['values'][$fVals[$valID]])
	) {
		return($fieldsAR[$fieldID]['values'][$fVals[$valID]]);
	} else {
		return(varValue($valID, $fVals));
	}
}

function oc_genFieldMessage(&$fsAR, &$fieldsAR, &$fVals=array()) {
	$fmsg = '';
	foreach ($fsAR as $fieldset => $fieldsetAR) {
		if ($fieldset == 'fs_authors') {
			for ($i=1; $i<=$GLOBALS['oc_authorNum']; $i++) {
				if (empty($fVals['name_last' . $i])) { continue; }
				$fmsg .= 'Author ' . $i . ":\n";
				foreach ($fieldsetAR['fields'] as $fieldID) {
					$fmsg .= '    ' . $fieldsAR[$fieldID]['short'] . ": " . oc_getFieldValue($fieldsAR, $fVals, $fieldID, $fieldID . $i) . "\n";
				}
				$fmsg .=  "\n\n";
			}
		} else {
			foreach ($fieldsetAR['fields'] as $fieldID) {
				if (preg_match("/^password/i", $fieldID) || ($fieldID == 'file') || empty($fieldsAR[$fieldID]['name'])) { continue; }
				$fmsg .= $fieldsAR[$fieldID]['short'] . ': ' . oc_getFieldValue($fieldsAR, $fVals, $fieldID) . "\n\n";
			}
		}
	}
	return($fmsg);
}

function oc_showFieldSet(&$fsAR, &$fieldsAR, &$fVals) {
	foreach ($fsAR as $fieldset => $fieldsetAR) {
		if (count($fieldsetAR['fields']) == 0) { continue; }	// skip fieldset if no fields included
		if (preg_match("/^fs_passwords$/i", $fieldset)) { continue; }	// skip password fs
		if ($fieldset == 'fs_authors') {	// display author fields
			for ($i=1; $i<=$GLOBALS['oc_authorNum']; $i++) {
				if (!isset($fVals['name_last' . $i])) { continue; }
				print '<tr><th>Author ' . $i . ':</th><td><table class="ocfields">';
				foreach ($fieldsetAR['fields'] as $fieldID) {
					if (!isset($fieldsAR[$fieldID])) { continue; }
					print '<tr><th>' . safeHTMLstr($fieldsAR[$fieldID]['short']) . ':</th><td>';
					if ($fieldID == 'email') {
						$e = oc_getFieldValue($fieldsAR, $fVals, $fieldID, $fieldID . $i);
						print '<a href="mailto:' . $e . '">' . safeHTMLstr($e) . '</a>';
					} else {
						print oc_getFieldValue($fieldsAR, $fVals, $fieldID, $fieldID . $i);
					}
					print "</td></tr>\n";
				}
				print "</table></td></tr>\n";
			}
		} else {	// not authors fieldset
			foreach ($fieldsetAR['fields'] as $fieldID) {
				if (!isset($fieldsAR[$fieldID])) { continue; }
				if (preg_match("/^(?:file|format)/", $fieldID)) { continue; }	// skip file/format fields
				print '<tr><th>' . safeHTMLstr($fieldsAR[$fieldID]['short']) . ':</th><td>' . nl2br(safeHTMLstr(trim(oc_getFieldValue($fieldsAR, $fVals, $fieldID)))) . "</td></tr>\n";
			}
		}
	}
}

function oc_checkField($fieldID, &$fieldsAR, &$saveFieldAR, &$err, $checkArrayValue=false) {
	if (oc_fieldEnabled($fieldID, $fieldsAR)) {
		if (isset($_POST[$fieldID]) && !empty($_POST[$fieldID])) {
			if ($checkArrayValue && is_array($fieldsAR[$fieldID]['values'])) {
				if ($fieldsAR[$fieldID]['usekey']) {
						if (isset($fieldsAR[$fieldID]['values'][$_POST[$fieldID]])) {
							$saveFieldAR[$fieldID] = "'" . safeSQLstr(trim($_POST[$fieldID])) . "'";
						}
				} elseif (in_array($_POST[$fieldID], $fieldsAR[$fieldID]['values'])) {
					$saveFieldAR[$fieldID] = "'" . safeSQLstr(trim($_POST[$fieldID])) . "'";
				}
			} else {
				$saveFieldAR[$fieldID] = "'" . safeSQLstr(trim($_POST[$fieldID])) . "'";
			}
		}
		
		if (!isset($saveFieldAR[$fieldID])) {
			if ( isset($fieldsAR[$fieldID]['required']) && $fieldsAR[$fieldID]['required'] ) {
				$err .= '<li>' . sprintf(oc_('%s field is required'), oc_($fieldsAR[$fieldID]['short'])) . '</li>';
			} else {
				$saveFieldAR[$fieldID] = "''";
			}
		}
	}
}

function oc_checkAuthorField($fieldID, $authorID, &$fieldsAR, &$saveFieldAR, &$err, $checkArrayValue=false) {
	if (oc_fieldEnabled($fieldID, $fieldsAR)) {
		if (isset($_POST[$fieldID . $authorID]) && !empty($_POST[$fieldID . $authorID])) {
			if ($checkArrayValue && is_array($fieldsAR[$fieldID]['values'])) {
				if ($fieldsAR[$fieldID]['usekey']) {
						if (isset($fieldsAR[$fieldID]['values'][$_POST[$fieldID . $authorID]])) {
							$saveFieldAR[$authorID][$fieldID] = "'" . safeSQLstr(trim($_POST[$fieldID . $authorID])) . "'";
						}
				} elseif (in_array($_POST[$fieldID . $authorID], $fieldsAR[$fieldID]['values'])) {
					$saveFieldAR[$authorID][$fieldID] = "'" . safeSQLstr(trim($_POST[$fieldID . $authorID])) . "'";
				}
			} else {
				$saveFieldAR[$authorID][$fieldID] = "'" . safeSQLstr(trim($_POST[$fieldID . $authorID])) . "'";
			}
		}
		
		if (!isset($saveFieldAR[$authorID][$fieldID])) {
			if ( isset($fieldsAR[$fieldID]['required']) && $fieldsAR[$fieldID]['required'] ) {
				//T: %1$d = author number, %2$s = field name
				$err .= '<li>' . sprintf(oc_('Author %1$d %2$s field is required'), $authorID, oc_($fieldsAR[$fieldID]['short'])) . '</li>';
			} else {
				$saveFieldAR[$authorID][$fieldID] = "''";
			}
		}
	}
}

